# -*- coding: iso-8859-1 -*-  

import os
from sgt.edr.edr import Edr
import sgt.edr.jsmath as jsmath
import codecs, json
from sgt.picnictd.picnictd import Picnictd

import smacases, haacases

calcdirs = []
for sitehr in ['ZNO/BD', 'ZNO/OPA', 'JS/EFF', 'JS/OPA', 'NL/BD', 'NL/OPA', 'SR/OPA', 'JO/OPA', 'WLB/MGL']:
    for dir in ['picnictd/SMA_FO']:
        calcdirs.append(os.path.join('/projects/SGT/SMA/', sitehr, dir))

for sitehr in ['ZNO/OPA', 'NL/OPA', 'JO/OPA']:
    for dir in ['picnictd/BE', 'picnictd/HAA', 'picnictd/LMA']:
        calcdirs.append(os.path.join('/projects/SGT/HAA', sitehr, dir))

calcdirs.append('/Space/SGT2/resultateordner/calc/')
for dir in 'HAA BE LMA'.split():
    calcdirs.append('/projects/SGT/HAA/GEN/OPA/stman/%s/' % dir)

import time

edr = Edr('edr_NAB-14-36', u'NAB 14-36: Provisorische Sicherheitsanalysen fr SGT Etappe 2: Elektronischer Daten- und Resultateordner (EDR) zu den Dosisberechnungen in NTB&nbsp;14&#8209;03')# (Stand: %s)' % time.strftime("%d.%m.%Y %H:%M"))

edr.co_lib()#(301)


codecs.open(os.path.join(edr.jsdata, 'edrtitle.js'), 'w', 'utf-8').write('edrtitle=%s;' % (json.dumps(edr.title)))


import dataviewer.doseplot
import os
import h5py
import math

def hdf5obj(calcdirs, h5filename):
    for calcdir in calcdirs:
        if os.path.exists(os.path.join(calcdir, h5filename+'.hdf5')):
            break    
    try:
        f = h5py.File(os.path.join(calcdir, h5filename+'.hdf5'), 'r')
    except:
        print 'cannot find', h5filename
        raise
    return f


def hdf5text(calcdirs, h5filename, aspect):
    for calcdir in calcdirs:
        if os.path.exists(os.path.join(calcdir, h5filename+'.hdf5')):
            break    
    try:
        f = h5py.File(os.path.join(calcdir, h5filename+'.hdf5'), 'r')
    except:
        print 'cannot read', h5filename, aspect
        raise
    return f[aspect].value


basename_formula = {}
def define_release(basename, formula, dcf, name_formula):
    #amountrate = "%s:AmountRate" % basename
    #name_formula[amountrate] = formula
    #name_formula["%s:ActivityRate" % basename] = 'PQ("%s").mul("opa_lis.Bq_pro_mol")' % amountrate
    #name_formula["%s:DoseRate" % basename] = 'PQ("%s").mul("%s")' % (amountrate, dcf)
    basename_formula[basename] = formula
    namedoserate = "%s:DoseRate" % basename
    formuladoserate = '(%s).mul("%s")' % (formula, dcf)
    name_formula[namedoserate] = formuladoserate

name_formula = jsmath.NameFormula()
#name_formula["convert.Bq_per_mol"] = 'PQ("opa_lis.Bq_pro_mol")'
#name_formula["convert.RTI_per_mol"] = 'PQ("opa_lis.mSvplus_pro_mol").mul(PQ(10, "mSv-1"))'

calc_coordinates = {}
case_descr = {}

# SMA

nametmpl = "%(reptype)s:%(site)s:%(hr)s:%(waste)s:%(kuerzel)s:%(path)s"

for reptype, site, hr, forwhat, nr, kuerzel, descr, calctmpls, bio, config in smacases.list_of_reptype_stite_hr_forwhat_nr_case_descr_calctmpl_bio_config[:]:
    #print calctmpls, bio, config
    if forwhat == 'Standortgebiet':
        hr = ''
    case_descr["%(reptype)s:%(site)s:%(hr)s:%(kuerzel)s" % locals()] = descr
    waste_calcs = {}
    allcalcs = []
    for calctmpl in calctmpls.split(','):
        calc = calctmpl % locals()
        waste_calcs.setdefault('SMA', []).append(calc) 
        calc_coordinates[calc] = smacases.config_settings[config]['coordinates']
        allcalcs.append(calc)            
    for path, tmpl in smacases.config_settings[config]['path_tmpl'].items():
        for waste, calcs in waste_calcs.items():
            formula = '.'.join([tmpl % locals() for calc in calcs])
            define_release(nametmpl % locals(), formula, bio+".btk", name_formula)

# HAA


for reptype, site, hr, kuerzel, descr, calcsBE, calcsHAA, calcsLMA, bio, config in haacases.list_of_reptype_site_hr_case_descr_calcBE_calcHAA_calcLMA_bio_config[:]:
    hr = ''
    case_descr["%(reptype)s:%(site)s:%(hr)s:%(kuerzel)s" % locals()] = descr
    waste_calcs = {}
    allcalcs = []
    for calcs, waste in zip([calcsBE, calcsHAA, calcsLMA], ['BE', 'HAA', 'LMA']):
        if not calcs:
            continue
        for calc in calcs.split(','):
            waste_calcs.setdefault(waste, []).append(calc)
            calc_coordinates[calc] = haacases.config_settings[config]['coordinates']
            allcalcs.append(calc)
            
    for path, tmpl in haacases.config_settings[config]['path_tmpl'].items():
        for waste, calcs in waste_calcs.items():
            formula = '.'.join([tmpl % locals() for calc in calcs])
            define_release(nametmpl % locals(), formula, bio+".btk", name_formula)
        waste = "BE+HAA+LMA"
        formula = '.'.join([tmpl % locals() for calc in allcalcs])
        define_release(nametmpl % locals(), formula, bio+".btk", name_formula)


# KOMBI (alle HAA Flle ausser NL + entspr. SMA Fall. Falls kein SMA Fall definiert ist -> SMA Referenzfall
for reptype, site, hr, kuerzel, descr, calcBE, calcHAA, calcLMA, bio, config in haacases.list_of_reptype_site_hr_case_descr_calcBE_calcHAA_calcLMA_bio_config[:]:
    if site == 'NL':
        continue
    hr = ''
    reptype = 'Kombi'
    case_descr["%(reptype)s:%(site)s:%(hr)s:%(kuerzel)s" % locals()] = descr
    haacalcs = [calcBE, calcHAA, calcLMA]
    for path, tmpl in haacases.config_settings[config]['path_tmpl'].items():
        try:
            sma = "SMA:%(site)s:%(hr)s:SMA:%(kuerzel)s:%(path)s:DoseRate" % locals()
            name_formula[sma]
        except:
            sma = "SMA:%(site)s:%(hr)s:SMA:RF:%(path)s:DoseRate" % locals()
            name_formula[sma]
        for waste, tmpl in [
            ('BE', 'PQ("HAA:%(site)s:%(hr)s:%(waste)s:%(kuerzel)s:%(path)s:DoseRate")'),
            ('HAA', 'PQ("HAA:%(site)s:%(hr)s:%(waste)s:%(kuerzel)s:%(path)s:DoseRate")'),
            ('LMA', 'PQ("HAA:%(site)s:%(hr)s:%(waste)s:%(kuerzel)s:%(path)s:DoseRate")'),
            ('SMA', 'PQ("%(sma)s")')
            ]:
            alreadydefinedformula = tmpl % locals()
            core = alreadydefinedformula[4:-2]            
            if core in name_formula:
                print core
                name_formula[(nametmpl % locals()) + ":DoseRate"] = alreadydefinedformula
        for waste, tmpl in [
            ('BE+HAA+LMA', 'PQ("HAA:%(site)s:%(hr)s:%(waste)s:%(kuerzel)s:%(path)s:DoseRate")'),
            ('BE+HAA+LMA+SMA', 'plus("HAA:%(site)s:%(hr)s:BE+HAA+LMA:%(kuerzel)s:%(path)s:DoseRate").plus("%(sma)s")')
            ]:
            name_formula[(nametmpl % locals()) + ":DoseRate"] = tmpl % locals()
            print [(nametmpl % locals()) + ":DoseRate"],  tmpl % locals()

            
codecs.open(os.path.join(edr.jsdata, 'case_description.js'), 'w', 'utf-8').write("case_description = %s;" % json.dumps(case_descr, indent=2))       

from pprint import pprint
#pprint(name_formula)

#
print name_formula.jsitems()

# creates all *js files for all data in all fomulas
lmacalc = 'HAA-GEN-OPA-stman-LMA-62-LMA'
becalc = 'HAA-GEN-OPA-stman-BE-62'
haacalc = 'HAA-GEN-OPA-stman-HAA-61'
lma = '((plus("%(lmacalc)s1_FO.cmi").plus("%(lmacalc)s2_FO.cmi")).mul(1E-2,"1/a"))' % locals()
be = '((plus("%(becalc)s-BES1.fmi").plus("%(becalc)s-BED2.fmi").plus("%(becalc)s-BED3.fmi")).mul(1E-5,"1/a"))' % locals()
haa = '((plus("%(haacalc)s-HAA1.gmi").plus("%(haacalc)s-HAA2.gmi")).mul(1E-4,"1/a"))' % locals()

def sl(calc):
    return '(%s).slice({"ge":PQ(1E6, "a")}).mul("dcf.msv_pro_mol").mul(0.73/1E6)' % calc #AN_14_03_Rev_1-R.btk


"""name_formula['HAA:JO::LMA:A10.6_M1:Aus Nahfeld:DoseRate'] = sl(lma)
name_formula['HAA:JO::BE:A10.6_M1:Aus Nahfeld:DoseRate'] = sl(be)
#name_formula['HAA:JO::HAA:A10.6_M1:Aus Nahfeld:DoseRate'] = sl(haa)
name_formula['HAA:JO::BE+HAA+LMA:A10.6_M1:Aus Nahfeld:DoseRate'] = sl(be + '.plus' + lma)# + '.plus' + haa)
"""

name_formula.to_json(edr.jsdata, calcdirs)
#name_formula.to_database(edr.jsdata, calcdirs, 'database.js', ['Lagertyp', 'Standortgebiet', 'Wirtgestein', 'Inventar', u'Krzel', 'Orte der Freisetzung', 'Observation'], edrtitle=edr.title, case_description=case_descr)

import vpac2svg

binfiles_svgfilename = {}

def make_nice_vpac_html(outputdir):
    calcname = os.path.basename(outputdir).replace('output_', '')
    htmlfilename = '%s.html' % os.path.join(outputdir, calcname)
    binfiles = vpac2svg.binfiles(os.path.join(os.path.dirname(outputdir), calcname))
    if binfiles in binfiles_svgfilename:
        svgfilename = binfiles_svgfilename[binfiles]
    else:
        vpac2svg.create_svg(os.path.join(os.path.dirname(outputdir), calcname), 'tmp.svg')
        svgfilename = "%s.svg" % hashlib.sha224(open('tmp.svg').read()).hexdigest()
        binfiles_svgfilename[binfiles] = svgfilename
        os.rename('tmp.svg', os.path.join(edr.htmldata, svgfilename))

    body = codecs.open(htmlfilename, 'r', 'latin-1').read()
    content = body.replace('<html>', '').replace('</html>', '')
    content = content.replace('<h2>Material Properties</h2>', '<iframe src="%s" width="500" height="500"></iframe> \n<h3>Material Properties</h3>' % svgfilename)
    content = content.replace("Permeability", "Hydr. conductivity")
    title = edr.title    
    breadcrumbs = ''#u'<a href="../edr.html">Startseite</a>'
    template = codecs.open('templates/vpac_data.html', 'r', 'utf-8').read()
    inputhtml = codecs.open(os.path.join(edr.htmldata, calcname + '.html'), 'w', 'utf-8')
    footer = ''#config.footer
    #for k,v in locals().items():
    #    if v.__class__ in (str,):
    #        print 'str:', k, v
    inputhtml.write(template % locals())

import hashlib, vpac2svg
import sgt.stman.stman as stmaninput

        
seen = set()

listof_targetSource = []

import re

emailandnofreigabe = re.compile("(\s*#\s*Freigabe)|(.*e-?mail)", re.IGNORECASE)

missing = open('missingvpachtml', 'w')
for jsitem in name_formula.jsitems():
    name, item = jsitem.split('.')
    if '-picnictd-' in name:
        if name in seen:
            continue
        f = hdf5obj(calcdirs, name)
        inputtxt = '\n'.join(["#%s:\n%s\n" % (item, f['inputfiles/%s' % item].value.decode('latin-1')) for item in f['inputfiles']])
        inputtxt = '\n'.join([l for l in inputtxt.splitlines() if not emailandnofreigabe.match(l)])
        codecs.open(os.path.join(edr.jsdata, "%s.js" % name), 'w', 'utf-8').write('define({"txt": %s});' % json.dumps(inputtxt))
        inputtxt = hdf5text(calcdirs, name, '/inputfiles/%s.in' % name).decode('latin-1')
        seen.add(name)
        try:
            p = Picnictd.parse_string(inputtxt)
        except:
            raise
        p.prepare_web()
        for src in p.stman_sourcedict.values():
            stmancalcname =  src.rsplit('.')[0]
            stmancalcbasename = os.path.basename(stmancalcname)
            prb = stmancalcname + '.prb'
            listof_targetSource.append((name, stmancalcbasename))
            if stmancalcbasename in seen:
                continue
            inputtxt = codecs.open(prb, 'r', 'latin-1').read()
            inputtxt = '\n'.join([l for l in inputtxt.splitlines() if not emailandnofreigabe.match(l)])
            codecs.open(os.path.join(edr.jsdata, "%s.js" % stmancalcbasename), 'w', 'utf-8').write('define({"txt": %s});' % json.dumps(inputtxt))
            try:
                s = stmaninput.Stman.parse(prb)
            except:
                raise#continue
            s.prepare()
            s.dbg = s.filename.replace('.prb', '.dbg')
            s.save_html(os.path.join(edr.htmldata, stmancalcbasename),
                        templatepath='templates/stman_data.html',
                        title=edr.title,
                        footer='')
            os.system('rm %s' % (os.path.join(edr.htmldata, stmancalcbasename)+'.input.html'))
            seen.add(stmancalcbasename)
        if p.vpac_sourcedict:
            vpacoutputdirs = sorted(set([src.rsplit('/', 1)[0] for src in p.vpac_sourcedict.values()]))
            if len(vpacoutputdirs) > 1:
                endings = ''.join(sorted(set([vpacoutputdir[-1] for vpacoutputdir in vpacoutputdirs])))
                print endings, vpacoutputdirs
                if endings in 'ABCDEFGHIJKLMNOPQ':
                    # although we have splitted the decay block leading to several calculations,
                    # we want to show the unsplitted input only
                    unsplittedvpacoutputdir = vpacoutputdirs[0][:-2]
                    unsplittedcalcname = os.path.basename(unsplittedvpacoutputdir).replace('output_', '')
                    unsplittedhtml = os.path.join(unsplittedvpacoutputdir, unsplittedcalcname)+'.html'
                    if os.path.exists(unsplittedhtml):                        
                        listof_targetSource.append((name, unsplittedcalcname))
                        vpacoutputdirs = [unsplittedvpacoutputdir]
                        #seen.add(unsplittedcalcname)                                                
                    else:
                        print >> missing, 'Unsplitted HTML output missing !!!!!!!!!!!!!!!!!', unsplittedhtml
                        print 'Unsplitted HTML output missing !!!!!!!!!!!!!!!!!', unsplittedhtml
                        
                    
            for vpacoutputdir in vpacoutputdirs:
                calcname = os.path.basename(vpacoutputdir).replace('output_', '')
                listof_targetSource.append((name, calcname))
                if calcname in seen:
                    continue
                html = os.path.join(vpacoutputdir, calcname)+'.html'
                if os.path.exists(html):
                    print 'Ok: ', html
                    try:
                        make_nice_vpac_html(vpacoutputdir)
                    except:
                        #raise
                        print missing, 'unexpected error in make_nice_html(', vpacoutputdir, ')'
                        print >> missing, 'unexpected error in make_nice_html(', vpacoutputdir, ')'
                else:
                    print >>missing, 'Missing !!!!!!!!!!!!!!!!!', html
                    print 'Missing !!!!!!!!!!!!!!!!!', html                
                seen.add(calcname)
            
        #create_sourceinfo(p)
        coordinates = calc_coordinates[name]
        #print 'check coordinates', name
        for legname, leg in p['LEG'].items():
            for j in 'FROM', 'TO':
                if leg[j] in coordinates or (legname in coordinates and j in coordinates[legname]):
                    continue
                else:
                    print name, 'coordinates missing  !!!!!!!!!!!:', leg[j], 'or', legname, j
                    1/0
                
        pdict = p.get_dict()
        pdict['junc_coords'] = coordinates        
        codecs.open(os.path.join(edr.jsdata, "%s.networks.js" % name), 'w', 'utf-8').write('define(%s);' % (
            json.dumps(pdict, indent=2)))
        seen.add(name) 
        
    
#
codecs.open(os.path.join(edr.jsdata, 'targetsource.js'), 'w', 'utf-8').write('listof_targetSource=(%s);' % (json.dumps(sorted(listof_targetSource), indent=2)))
missing.close()

import os.path
dirname = os.path.dirname(sys.argv[0])
targetdirname = edr.py

for basename in [smacases.__file__.replace('.pyc', '.py'), haacases.__file__.replace('.pyc', '.py'), os.path.basename(sys.argv[0])]:
    try:
        os.system('cp %(dirname)s%(basename)s %(targetdirname)s' % locals())
    except:
        pass

edr.zip(dirname="%s_%s" % (edr.directory, time.strftime("%d%m%y")), excludedirs=['exports'])


