var dv = dv || {};

// 1 pt = 1/72 inch, 1px = 0.1 mm

(function(d3, dm) {
    "use strict";
    dv.pt2px = 3.528;

    dv.pt = function(px, dpi){
        // convention:  1 px = 0.1 mm
        return px * dpi/254;
    };

    dv.supernumbers = function(text, s){
        var i, tspan, parts = s.replace(/(\W?\d+)/g, ':!:$1:!:').split(':!:'),
            dy = text.attr("dy");
        text.attr("dy", null);
        text.html('');
        for (i = 0; i < parts.length; i = i + 2){
            tspan = text
                .append('tspan')
                .text(parts[i])
            ;
            if (i>0){
                tspan.attr("dy", "0.5em").style("font-size", 'normal');
            } else {
                tspan.attr("dy", dy);
            }
            if (parts[i+1] !== undefined) {
                text
                    .append('tspan')
                    .attr("dy", "-0.5em")
                    .style("font-size", 'smaller')
                    .text(parts[i+1])
                ;
            }
        }
    };

    dv.get_scientificformatter = function (fontsizept){
        return function(){
            var that = d3.select(this),
                mantexp = that.text().split('e'),
                dy = that.attr("dy"),
                smaller = fontsizept*dv.pt2px*0.7;
            if (dy.endsWith("em")){
                dy = parseFloat(that.attr("dy"))*fontsizept*dv.pt2px;
            }
            that.style({'font-size': fontsizept*dv.pt2px+"px"});
            that.attr("dy", null);
            //console.log('mantexp', mantexp);
            if (mantexp.length === 2) {
                that.text("");
                if (mantexp[0] === "1") {
                    that.append("tspan")
                        .attr("dy", dy)
                        .text("10");
                } else {
                    that.append("tspan")
                        .text(mantexp[0]+  ' x 10');
                }
                that.append("tspan")
                    .attr("font-size", smaller)
                    .attr("dy", -smaller*0.7)
                    .text(mantexp[1].replace('+', ''));
            }
        };
    };

    dv.copybb = function (txt, rect){
        var bb = txt.node().getBBox();
        rect.attr("x", bb.x);
        rect.attr("y", bb.y);
        rect.attr("width", bb.width);
        rect.attr("height", bb.height);
    };


    dv.natdoserate = function(doseratechart, betrachtungszeitraum, label){
        var fontsizept = 10;
        var x1 = doseratechart.leftMargin(),
            x2 = doseratechart.width() - doseratechart.rightMargin()
            ;
        var yunits = doseratechart.yUnits();
        var yscale = doseratechart.yScale();
        var y1 = yscale(dm.PQ(20.0, 'mSv/a').inUnitsOf(yunits).mag);
        var y2 = yscale(dm.PQ(1, 'mSv/a').inUnitsOf(yunits).mag);
        var xtext = (x1 + (betrachtungszeitraum === undefined ? x2 : doseratechart.xScale()(betrachtungszeitraum.inUnitsOf(doseratechart.xUnits()).mag)))/2;
        var ytext = (y1 + y2)/2;
        var groupnaturaldose = doseratechart.plotarea().selectAll("g.naturaldose")
                .data([null]);
        groupnaturaldose.enter()
            .call(function(sel){
                var g = sel.append('g').attr('class', 'naturaldose');
                g.append('rect')
                    .style({fill: '#ffe596', stroke:'black', 'stroke-width': '1px'});
                g.append("text")
                    .attr("text-anchor", 'middle')
                    .style("font-size", fontsizept*dv.pt2px+"px")
                    .attr('dy', 0.3*fontsizept*dv.pt2px+"px")
                    .text(label || 'Natürliche Strahlenexposition in der Schweiz');
            });
        groupnaturaldose.selectAll("text")
                .attr("x", xtext)
                .attr("y", ytext);
        groupnaturaldose.selectAll("rect")
            .attr("x", x1)
            .attr("y", y1)
            .attr("width", x2 - x1)
            .attr("height", y2 - y1);
    };

    dv.ensig03 = function(doseratechart){
        var fontsizept = 10;
        var yunits = doseratechart.yUnits();
        var yscale = doseratechart.yScale();
        var y = yscale(dm.PQ(0.1, 'mSv/a').inUnitsOf(yunits).mag);
        var groupensig03 = doseratechart.plotarea().selectAll("g.ensig03")
            .data([null]);
        groupensig03.enter()
            .call(function(sel){
                var g = sel.append('g')
                    .attr('class', 'ensig03');
                g
                    .append('line')
                    .style({fill: 'none', stroke:'black', 'stroke-width': '4px'});
                var txt = g
                    .append("text")
                    .style("font-size", fontsizept*dv.pt2px+"px")
                    .attr("text-anchor", 'start');
                txt.append('tspan')
                    .attr('dx', fontsizept*dv.pt2px+"px")
                    .attr('dy', -0.5*fontsizept*dv.pt2px)
                    .text("ENSI-G03: 0.1 mSv a");
                txt.append('tspan')
                    .attr('dy', -0.5*fontsizept*dv.pt2px)
                    .style("font-size", "smaller")
                    .text("-1");
            });
        groupensig03.selectAll("line")
            .attr("x1", doseratechart.leftMargin())
            .attr("x2", doseratechart.width() - doseratechart.rightMargin())
            .attr("y1", y)
            .attr("y2", y);
        groupensig03.selectAll("text")
            .attr("x",  doseratechart.leftMargin())
            .attr("y",  y);
    };

    dv.stsv = function(doseratechart){
        var fontsizept = 10;
        var yunits = doseratechart.yUnits();
        var yscale = doseratechart.yScale();
        var y = yscale(dm.PQ(0.01, 'mSv/a').inUnitsOf(yunits).mag);
        var groupstsv = doseratechart.plotarea().selectAll("g.StSV")
            .data([null]);
        groupstsv.enter()
            .call(function(sel){
                var g = sel.append('g')
                    .attr('class', 'StSV');
                g
                    .append('line')
                    .style({fill: 'none', stroke:'black', 'stroke-width': '4px', 'stroke-dasharray': '20px 20px'});
                var txt = g
                    .append("text")
                    .style("font-size", 10*dv.pt2px+"px")
                    .attr("text-anchor", 'start');
                txt.append('tspan')
                    .attr('dx', fontsizept*dv.pt2px)
                    .attr('dy', -0.5*fontsizept*dv.pt2px)
                    .text("StSV: 0.01 mSv a");
                txt.append('tspan')
                    .attr('dy', -0.5*fontsizept*dv.pt2px)
                    .style("font-size", "smaller")
                    .text("-1");
            });
        groupstsv.selectAll("line")
            .attr("x1", doseratechart.leftMargin())
            .attr("x2", doseratechart.width() - doseratechart.rightMargin())
            .attr("y1", y)
            .attr("y2", y);
        groupstsv.selectAll("text")
            .attr("x",  doseratechart.leftMargin())
            .attr("y",  y);
    };


    dv.shadow = function(chart, x, y){
        var x0 = chart.leftMargin(),
            y0 = chart.height()- chart.bottomMargin(),
            y1,
            x2 = chart.width() - chart.rightMargin(),
            x1 = x === null? x2: chart.xScale()(x.inUnitsOf(chart.xUnits()).mag),
            y2 = chart.topMargin()
            ;
        if (y){
            y1 = chart.yScale()(y.inUnitsOf(chart.yUnits()).mag)
        } else {
            y1 = y0;
        }
        var shadowpath = chart.plotarea().selectAll("path.shadow")
            .data([null]);
        shadowpath
            .enter()
            .append('path')
            .attr("class", "shadow")
            .style({fill: '#DCDCDC', stroke:'black', 'stroke-width':'1px', 'stroke-opacity': 1, 'fill-opacity': 0.75})
        ;
        shadowpath
            .attr("d", "M"+x0+" "+y1+"L"+x1+" "+y1+"L"+x1+" "+y2+"L"+x2+" "+y2+"L"+x2+" "+y0+"L"+x0+" "+y0+"Z")
        ;
    };

    dv.marker = function(symbol, size, orient){
        var symbol_d = {
            'square': function(r) {
                var ref = r * 2;
                return "M " + ref+","+ref
                    + "m " + -r + "," + -r
                    + "h " + ref
                    + "v " + ref
                    + "h " + -ref
                    + "z";
            },
            'triangle-down': function(ry) {
                var rx = ry / (Math.sqrt(3) / 2),
                    ref = 2* rx;
                return "M"+ ref+","+(ref+ry)
                    + " l " + rx +"," + -(2*ry)
                    + " h" + -(2*rx)
                    + "z";
            },
            'triangle-up': function(ry){
                var rx = ry / (Math.sqrt(3) / 2),
                    ref = 2* ry;
                return "M"+ ref+","+(ref-ry)
                    + " l " + rx +"," + (2*ry)
                    + " h" + -(2*rx)
                    + "z";
            },
            'diamond': function(ry){
                var rx = ry/2.0;
                return "M" + (2*ry) + "," + ry
                    + "l"+ rx + "," + ry
                    + " " + (-rx) + "," + ry
                    + " " + (-rx) + "," + (-ry)
                    + "z";
            },
            'circle': function(r){
                return "M"+(2*r)+"," + r
                    + "a" + r + "," + r + " 0 0,0 0," + (2*r)
                    + "a" + r + "," + r + " 0 0,0 0," + (-2*r)
                    + "Z";
            },
            'cross': function(r){
                return "M" + r + "," + r
                    + "L" + (3*r) + "," + (3*r)
                    + "M" + r + "," + (3*r)
                    + "L" + (3*r) + "," + r;
            },
            'plus': function(r){
                return "M" + r + "," + (2*r)
                    + "H" + (3*r)
                    + "M" + (2*r) + "," + r
                    + "V" + (2*r);
            },
            'square45': function(d){
                var l = Math.sqrt(2) * d;
                return 'M' + (2*d - l) + "," + (2*d)
                    + "l" + l + "," + l
                    + "l" + l + "," + (-l)
                    + "l" + -l + "," + (-l)
                    +"z";
            }
        };
        return {
            d: (symbol_d[symbol] === undefined ? symbol_d.plus : symbol_d[symbol])(size),
            attr: {
                refX: 2*size,
                refY: 2*size,
                markerHeight: 4*size,
                markerWidth: 4*size,
                markerUnits: "userSpaceOnUse",
                orient: orient === undefined ? "0" : orient,
                viewBox: "0 0 " + (4*size) + " " + (4*size)
            }
        };
    };


    dv.legendfactory = function(){
        var fontsizept = 8,// fontsize in pt (1pt = 1/72 inch = 0.3528 mm = 3.528 px)
            fontsize = fontsizept*dv.pt2px,
            linespacing = 2*dv.pt2px,
            pathlength = 80,
            verticalspace = fontsizept*dv.pt2px,
            horizontalpadding = 30,
            charwidthfactor = 0.5
            ;

        function wordwrapfactory(width, cut) {
            width = width || 75;
            cut = cut || false;
            var regex = '.{1,' +width+ '}(\\s|$)' + (cut ? '|.{' +width+ '}|.+$' : '|\\S+?(\\s|$)');
            return function(str){
                if (!str) {return str;}
                return str.match(new RegExp(regex, 'g'));
            };
        }

        function Labelbox(labellines){
            this.labellines = labellines || [];
            this._width = Math.max.apply(null, (this.labellines.map(function(line){return line.length;}))) * fontsize * charwidthfactor;
        }

        Labelbox.prototype.width = function(){
            return this._width;
        };

        Labelbox.prototype.height = function(){
            return this.numberoflines() * (fontsize + linespacing);
        };

        Labelbox.prototype.numberoflines = function(){
            return this.labellines.length;
        };


        function Legend(items){
            this.items = items;
        }



        Legend.prototype.construct = function(availablewidth, columns){
            return this.construct_(availablewidth, columns, "line");
        };

        Legend.prototype.constructrect = function(availablewidth, columns){
            return this.construct_(availablewidth, columns, "rect");
        };

        Legend.prototype.construct_ = function(availablewidth, columns, symbol){
            var availablelabelboxwidth = availablewidth/(columns || 1),
                x0, y0;
            var labelbox,
                cumheight= 0,
                width = 0,
                height= 0,
                edgelength = pathlength/2;
            var labeloffset = pathlength + horizontalpadding,
                pathoffset = horizontalpadding/2;
            var maxlabelwidth = availablelabelboxwidth - labeloffset,
                maxnumbercharsperline = Math.floor(maxlabelwidth/(fontsize * charwidthfactor));
            var wordwrap = wordwrapfactory(maxnumbercharsperline);
            var ic, idx, lastidx, heights = [];
            this.items.forEach(function(item){
                    item.subset.labelbox = new Labelbox(wordwrap(item.subset.label));
                    var height = item.subset.labelbox.height() + verticalspace;
                    cumheight += height;
                    heights.push(height);
                }
            );

            var st = dm.stacking.Stacking(heights).limitnumberofstacks(columns),
                confs = st.confs();

            this.items.forEach(function(item, i){
                var position = [confs[i].istack * availablelabelboxwidth + horizontalpadding/2+ labeloffset, confs[i].start],
                    labelbox = item.subset.labelbox
                    ;
                labelbox.position = position;
                x0 = confs[i].istack * availablelabelboxwidth + pathoffset;
                y0 = confs[i].start+labelbox.height()/2 + fontsize/3;
                if (symbol === "line"){
                    item.subset.path = "M" + x0 + " " + y0 + " h" + pathlength/2 + " h" + pathlength/2;
                } else if (symbol === "rect"){
                    item.subset.path = "M" + x0 + " " + (y0 + edgelength/2) + " h" + edgelength + " v-" + edgelength + " h-" +edgelength + " Z";
                }
                width = Math.max(width, position[0] + labelbox.width());
                height = Math.max(height, position[1] + labelbox.height());
            });
            return [width, height+100];
        };

        Legend.prototype.draw = function(ele){
            var g = ele//.append("svg")
                .selectAll('g')
                .data(this.items)
                .enter()
                .append("g");
            g.append("path")
                .each(function(d){
                    var path = d3.select(this);
                    path
                        .attr('class', function(d){return "total " + d.subset.cssclasses.join(' ');})
                        .style(d.subset.linestyle)
                        .attr('d', function(d){return d.subset.path;})
                    ;
                    if (d.subset.marker){
                        var markerurl = "url(#"+d.subset.cssclasses.join()+")";
                        path
                            .attr('marker-mid', markerurl)
                            //.attr('marker-start', markerurl)
                            //.attr('marker-end', markerurl)
                        ;
                    }

                });

            var labeltext = g.append("text")
                .style({font:'Arial', 'font-size': fontsize+"px"})
                .attr('class', function(d){return "total " + d.subset.cssclasses.join(' ');})
                .attr("x",function(d){return d.subset.labelbox.position[0];})
                .attr("y",function(d){return d.subset.labelbox.position[1];});

            labeltext.selectAll("tspan")
                .data(function(d){return d.subset.labelbox.labellines;})
                .enter()
                .append("tspan")
                .attr('font-size', fontsize+"px")
                .text(function(d){return d;})
                .attr("x", function(){return this.parentNode.__data__.subset.labelbox.position[0];})
                .attr("dy", fontsize + linespacing);
        };

        Legend.fontsize = function(_){
            if (!arguments.length) {
                return fontsize;
            }
            fontsize = _;
            return Legend;
        };

        Legend.linespacing = function(_){
            if (!arguments.length) {
                return linespacing;
            }
            linespacing = _;
            return Legend;
        };

        Legend.pathlength = function(_){
            if (!arguments.length) {
                return pathlength;
            }
            pathlength = _;
            return Legend;
        };

        Legend.horizontalpadding = function(_){
            if (!arguments.length) {
                return horizontalpadding;
            }
            horizontalpadding = _;
            return Legend;
        };

        Legend.verticalspace = function(_){
            if (!arguments.length) {
                return verticalspace;
            }
            verticalspace = _;
            return Legend;
        };

        return Legend;
    };


}(window.d3, window.dm));
