/**
 * Created with PyCharm.
 * User: a404799
 * Date: 20.09.13
 * Time: 09:30
 * To change this template use File | Settings | File Templates.
 */



/*
Objectfactory
used to create javascript objects with properties corresponding to attributes
 */
var dm = dm || {};

(function(){
    "use strict";
    function Objectfactory(){
        var allpropertynames = [],
            sep = '-',
            Of = {};

        Of.parse = function(name){
            var i,
                parts = name.split(sep);
            var obj = {name: name};
            for (i=0; i< parts.length; ++i){
                obj[allpropertynames[i]] = parts[i];
            }
            return obj;
        };

        Of.getstr = function(obj, propertynames, joinstr){
            joinstr = joinstr || sep;
            var mykeys = propertynames.length > 0 ? propertynames: allpropertynames;
            return mykeys.map(function(key){return obj[key];}).join(joinstr);
        };

        Of.allpropertynames = function(_){
            if (!arguments.length) {return allpropertynames;}
            allpropertynames = _;
            return Of;
        };

        Of.sep = function(_){
            if (!arguments.length) {return sep;}
            sep = _;
            return Of;
        };

        return Of;
    }
    dm.Objectfactory = Objectfactory;
}());

dm.grouper = function(){
    "use strict";
    var listof_item_length = [],
        last,
        lastitem_length
        ;
    return {
        add: function(item){
            if (item === last){
                lastitem_length[1] = lastitem_length[1] + 1;
            } else {
                lastitem_length = [item, 1];
                listof_item_length.push(lastitem_length);
                last = item;
            }
        },
        listof_item_length: function(){
            return listof_item_length;
        },
        listof_item_start_stop: function(){
            var arr = [],
                start, stop;
            listof_item_length.forEach(function(item_length, idx){
                if (idx ===0){
                    start = 0;
                }
                stop = start + item_length[1];
                arr.push([item_length[0], start, stop]);
                start = stop;
            });
            return arr;
        }
    };
};



/*
Pattern
 Used to store for a given attribute the value pattern of the items in a list
 e.g. if we push each value of ['c', 'a', 'b', 'a', 'c'] in that order to a pattern,
 it has the following attributes:
      - valueids: [0, 1, 2, 1, 0]
      - values ['c', 'a', 'b']
      - value_id: {a: 1, b: 2, c: 0}
      - distictvalues: 3 (i.e. the length of the values attribute)
 */
function Pattern(propertyname){
    "use strict";
    this.propertyname = propertyname;
    this.value_id = {};
    this.values = [];
    this.valueids = [];
    this.distinctvalues = 0;
}

Pattern.prototype.push = function(value) {
    "use strict";
    var id,
        valuestr = value.toString();
    id = this.value_id[valuestr];
    if (id === undefined) {
        id = this.value_id[valuestr] = this.distinctvalues++;
        this.values.push(value);
    }
    this.valueids.push(id);
};

Pattern.prototype.toString = function() {
    "use strict";
    return this.valueids.toString();
};



/*
Objects
 A list of objects.
 The objects are analysed to identify characteristic and common properties.

 chartdata() returns a list of items each with two properies:
  - global: the original object
  - subset: An object holding idx, cssclasses, label and linestyle

 */
(function(){
    "use strict";
    function Objects(){
        var i,
            objects = [],
            patterns,
            characteristicpropertygroup,
            characteristicpropertygroups,
            characteristicproperties,
            characteristicpatterns,
            commonproperties,
            commonproperty_value = {},
            objectfactory = dm.Objectfactory(),
            stylingpreferences = [],
            linestyle,
            pattern,
            barchartattributes,
            f = {}
        ;

        function update(obj, other){
            var key;
            for (key in other){
                if (other.hasOwnProperty(key)){
                    obj[key] = other[key];
                }
            }
        }

        f.objects = function(_){
            if (!arguments.length) {return objects.slice();}
            objects = _;

            var ps_patterns = {};
            patterns = [];
            characteristicpropertygroups = [];
            characteristicproperties = [];
            characteristicpatterns = [];
            commonproperties = [];

            objectfactory.allpropertynames().forEach(function(propertyname){
                patterns.push(new Pattern(propertyname));
            });

            objects.forEach(function(obj){
                patterns.forEach(function(pattern){
                    pattern.push(obj[pattern.propertyname]);
                });
            });

            patterns.forEach(function(pattern){
                var ps = pattern.toString();
                if (ps_patterns[ps]){
                    ps_patterns[ps].push(pattern);
                } else {
                    ps_patterns[ps] = [pattern];
                }
            });

            function pushcommon(pattern){
                commonproperties.push(pattern.propertyname);
                commonproperty_value[pattern.propertyname] = pattern.values[0];
            }

            function pushcharacteristic(pattern){
                characteristicproperties.push(pattern.propertyname);
                characteristicpropertygroup.push(pattern.propertyname);
            }
            var ps;
            for (ps in ps_patterns){
                if (ps_patterns.hasOwnProperty(ps)){
                    patterns = ps_patterns[ps];
                    if (patterns[0].distinctvalues === 1) {
                        patterns.forEach(pushcommon);
                    } else {
                        characteristicpatterns.push(patterns);
                        characteristicpropertygroup = [];
                        patterns.forEach(pushcharacteristic);
                        characteristicpropertygroups.push(characteristicpropertygroup);
                    }
                }
            }
            return f;
        };

        f.chartdata = function(attrnamesifoneonly){
            characteristicpatterns.sort(function(a, b){return b[0].distinctvalues - a[0].distinctvalues;});
            var numberobjectstyles = Math.min(characteristicpatterns.length, stylingpreferences.length);
            return objects.map(function(obj, subsetidx){
                linestyle = {fill: "none", stroke: "black", "stroke-width": 5};
                for(i=0; i < numberobjectstyles; ++i){
                    pattern = characteristicpatterns[i][0];
                    update(linestyle, stylingpreferences[i].get(pattern.valueids[subsetidx]));
                }
                return {
                    global: obj,
                    subset: {
                        idx: subsetidx,
                        cssclasses: characteristicpatterns.map(function(patterns, idx){return 'c' + idx + '-' + patterns[0].valueids[subsetidx];}),
                        label: objectfactory.getstr(obj, characteristicproperties.length > 0 ? characteristicproperties : (attrnamesifoneonly === undefined ? [] :attrnamesifoneonly), ' '),
                        linestyle: linestyle,
                        visible: 'total'
                    }
                };
            });
        };

        f.chartdatafromkeyvalue = function(attrname, attrvalue_linestyle){
            attrname = typeof attrname === 'string' ? function(obj){return obj[attrname];} : attrname;
            return objects.map(function(obj, subsetidx){
                linestyle = {fill: "none", stroke: "black", "stroke-width": 5};
                update(linestyle, attrvalue_linestyle[attrname(obj)].linestyle);
                return {
                    global: obj,
                    subset: {
                        idx: subsetidx,
                        cssclasses: characteristicpatterns.map(function(patterns, idx){return 'c' + idx + '-' + patterns[0].valueids[subsetidx];}),
                        label: objectfactory.getstr(obj, characteristicproperties.length > 0 ? characteristicproperties : (attrnamesifoneonly === undefined ? [] :attrnamesifoneonly), ' '),
                        linestyle: linestyle,
                        marker: attrvalue_linestyle[attrname(obj)].marker,
                        visible: 'total'
                    }
                };
            });
        };

        f.propertylists = function(){

        };

        f.propertydata = function(){
            return objects;
        };

        f.objectfactory = function(_){
            if (!arguments.length) {return objectfactory;}
            objectfactory = _;
            return f;
        };

        f.stylingpreferences = function(_){
            if (!arguments.length) {return stylingpreferences;}
            stylingpreferences = _;
            return f;
        };

        f.commonproperties = function(){
            return commonproperties;
        };

        f.characteristicproperties = function(){
            return characteristicproperties;
        };

        f.characteristicpropertygroups = function(){
            return characteristicpropertygroups;
        };

        f.title = function(getstr){
            return (getstr || objectfactory.getstr)(commonproperty_value, commonproperties);
        };

        return f;
    }
    dm.Objects = Objects;
}());




function Stylepool(attribute, values){
    this.attribute = attribute;
    this.values = values || [];
    this.current = 0;
}

Stylepool.prototype.size = function(){
    return this.values.length;
};

Stylepool.prototype.next = function(){
    return this.get(this.current++);
};

Stylepool.prototype.get = function(i){
    var style = {};
    if(this.attribute === undefined) {
        return {};
    }
    style[this.attribute] = this.values[i % this.size()];
    return style;
};
